@@ -1,3 +1,39 @@
+/*
+ * P3Scan v2.3.2
+ *
+ * (C) 2003-2005 by Jack S. Lai <laitcg@cox.net>
+ *
+ * It's intent is to provide a follow on program to POP3-Virusscan-Proxy 0.4
+ * by Folke Ashberg <folke@ashberg.de>.
+ *
+ * It is based upon his program but provides numerous changes to include
+ * scanning pop3 mail for spam, hardening the program, addaption to todays
+ * email environment, and many other changes.
+ *
+ * The initial release of p3scan includes patches made and submitted to the
+ * original project but were never incorporated. Please see the README for
+ * further information.
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *
+ * This program is released under the GPL with the additional exemption that
+ * compiling, linking, and/or using OpenSSL is allowed."
+ * (http://www.openssl.org/support/faq.html#LEGAL2)
+ *
+ */
+
 /*
  *
  * parsefile.c
@@ -7,7 +43,7 @@
  * $Id: parsefile.c,v 1.1.1.1 2003/01/23 12:41:40 vtoroman Exp $
  *
  * parsefile.c provides functions for parsing text with replacing keywords.
- * it uses my getline.c for string handling
+ * it uses my getlinep3.c for string handling
  *
  * This stuff provides functions for linehandlin on file descriptors,
  * especially using netork sockets, because the getline function is
@@ -37,13 +73,18 @@
 #include <string.h>
 #include <sys/stat.h>
 #include "parsefile.h"
-#include "getline.h"
+#include "getlinep3.h"
+#include "p3scan.h"
+
+extern void * w_malloc(size_t bytes);
+extern void w_free(void *f_address);
 
 #define NONULL(x) ( x==NULL ? "" : x) /* this is nice, found in the mutt code */
 
 struct paramlist * paramlist_init(void){
    struct paramlist *pl;
    pl=malloc(sizeof(struct paramlist));
+   //--pl=w_malloc(sizeof(struct paramlist));
    pl->name=NULL;
    pl->value=NULL;
    pl->next=NULL;
@@ -57,6 +98,9 @@ void paramlist_uninit(struct paramlist ** p){
       if ((*p)->name) free((*p)->name);
       if ((*p)->value) free((*p)->value);
       free(*p);
+      //--if ((*p)->name) w_free((*p)->name);
+      //--if ((*p)->value) w_free((*p)->value);
+      //--w_free(*p);
       *p=next;
    }
 }
@@ -76,6 +120,8 @@ int paramlist_set(struct paramlist * p, char * name, char * value){
    }
    if (p->name) free(p->name);
    if (p->value) free(p->value);
+   //--if (p->name) w_free(p->name);
+   //--if (p->value) w_free(p->value);
    p->name=strdup(name);
    p->value= value ? strdup(value) : NULL;
    return 0;
@@ -133,6 +179,7 @@ int paramlist_strreplace(char ** dst, char * in, struct paramlist * params){
             }
             rl_next=rl;
             rl=malloc(sizeof(struct replacelist));
+            //--rl=w_malloc(sizeof(struct replacelist));
             if (rl_last) rl_last->next=rl;
             else rl_first=rl;
             rl->next=rl_next;
@@ -154,6 +201,7 @@ int paramlist_strreplace(char ** dst, char * in, struct paramlist * params){
    }
    if (replacecount>0){
       *dst=malloc(len+1);
+      //--*dst=w_malloc(len+1);
       (*dst)[len]='\0';
       rl=rl_first;
       pos=0;
@@ -179,13 +227,15 @@ int paramlist_strreplace(char ** dst, char * in, struct paramlist * params){
       rl_last=rl;
       rl=rl->next;
       free(rl_last);
+      //--w_free(rl_last);
    }
    return replacecount;
 }
 
 int parsefile(char * infile, char * outfile , struct paramlist * params, int leading){
    int in, out, ret;
-   if ( (in=open(infile, O_RDONLY))<0) return 1;
+   //if ( (in=open(infile, O_RDONLY))<0) return 1;
+   if ( (in=open(infile, O_RDONLY))<0) return -1;
    /** if file already exists just send that file and don't do any parsing
      * prim vt */
    if ( (out=open(outfile,O_RDONLY))>=0){
@@ -206,13 +256,14 @@ int parsefds(int in, int out , struct paramlist * params, int leading){
    char * replaced;
    struct linebuf *l;
    if ((l=linebuf_init(4096))==NULL) return 1;
-   while ( (res=getline(in, l))>=0){
+   while ( (res=getlinep3(in, l))>=0){
       if (l->linelen >=0 ){
          replacements=paramlist_strreplace(&replaced, l->line, params);
          if (writeline(out, leading, replaced)){
             //error
          }
          if (replacements>0) free(replaced);
+         //--if (replacements>0) w_free(replaced);
       }
    }
    linebuf_uninit(l);
