@@ -62,6 +62,8 @@ struct MIMEH_globals {
 
 	char appledouble_filename[_MIMEH_STRLEN_MAX +1];
 
+	char subject[_MIMEH_STRLEN_MAX +1];
+
 	char *headerline;
 	char *headerline_original;	// Holds the original header-form without decoding.
 	int save_headers;
@@ -2427,8 +2429,16 @@ int MIMEH_parse_generic( char *header_name, char *header_value, struct MIMEH_hea
 \------------------------------------------------------------------*/
 int MIMEH_parse_subject( char *header_name, char *header_value, struct MIMEH_header_info *hinfo )
 {
-	return MIMEH_parse_generic( header_name, header_value, hinfo, "subject", hinfo->subject, sizeof(hinfo->subject)  );
+	int result = 0;
+	result = MIMEH_parse_generic( header_name, header_value, hinfo, "subject", hinfo->subject, sizeof(hinfo->subject)  );
+	snprintf(glb.subject, sizeof(glb.subject),"%s", hinfo->subject);
+
+	return result;
 }
+
+
+
+
 /*-----------------------------------------------------------------\
  Function Name	: MIMEH_parse_date
  Returns Type	: int
