@@ -19,6 +19,7 @@
 #include <errno.h>
 #include <syslog.h>
 #include <signal.h>
+#include <dirent.h>
 
 #include "buildcodes.h"
 #include "logger.h"
@@ -43,6 +44,7 @@ struct RIPMIME_globals
 	int timeout;
 	int quiet;
 	int verbose_defects;
+	int verbose;
 };
 
 //-b [blankzone file name] : Dump the contents of the MIME blankzone to 'filename'
@@ -51,57 +53,59 @@ struct RIPMIME_globals
 char defaultdir[] = ".";
 struct RIPMIME_globals *ripmime_globals = NULL;
 
-char version[] = "v1.4.0.3 - Dec 17, 2004 (C) PLDaniels http://www.pldaniels.com/ripmime";
-char help[] = "ripMIME -i <mime file> -d <directory>\
-	[-p prefix] [-e [header file]] [-vVh] [--version]\
-	[--no_nameless] [--unique_names [--prefix|--postfix|--infix]]\
-	[--paranoid] [--mailbox] [--formdata] [--debug]\
-	[--no-tnef] [--no-quotedprintable] [--no-uudecode]\n\
-Options available :\n\
-	-i : Input MIME encoded file (use '-' to input from STDIN)\n\
-	-d : Output directory\n\
-	-p : Specify prefix filename to be used on files without a filename (default 'text')\n\
-	-e [headers file name] : Dump headers from mailpack (default '_headers_')\n\
-	-v : Turn on verbosity\n\
-	-q : Run quietly, do no report non-fatal errors\n\
-	\n\
-	--verbose-contenttype : Turn on verbosity of file content type\n\
-	--verbose-oldstyle : Uses the v1.2.x style or filename reporting\n\
-	--verbose-defects: Display a summary of defects in the email\n\
-	--stdout : All reporting goes to stdout (Default)\n\
-	--stderr : All reporting goes to stderr\n\
-	--syslog : All reporting goes to syslog\n\
-	\n\
-	--no-paranoid : [ Deprecated ] Turns off strict ascii-alnum filenaming\n\
-	--paranoid: Converts all filenames to strict 7-bit compliance\n\
-	--name-by-type: Saves a given attachment by its content-type if it has no other name\n\
-	--no-nameless : Do not save nameless attachments\n\
-	--overwrite : Overwrite files if they have the same name on extraction\n\
-	--unique-names : Dont overwrite existing files (This is the default behaviour)\n\
-	--prefix : rename by putting unique code at the front of the filename\n\
-	--postfix : rename by putting unique code at the end of the filename\n\
-	--infix : rename by putting unique code in the middle of the filename\n\
-	\n\
-	--mailbox : Process mailbox file\n\
-	--formdata : Process as form data (from HTML form etc)\n\
-	\n\
-	--no-tnef : Turn off TNEF/winmail.dat decoding\n\
-	--no-ole : Turn off OLE decoding\n\
-	--no-uudecode : Turns off the facility of detecting UUencoded attachments in emails\n\
-	--no-quotedprintable : Turns off the facility of decoding QuotedPrintable data\n\
-	--no-doublecr : Turns off saving of double-CR embedded data\n\
-	--no-mht : Turns off MHT (a Microsoft mailpack attachment format ) decoding\n\
-	--no-multiple-filenames : Turns off the multiple filename exploit handling\n\
-	\n\
-	--disable-qmail-bounce : Turns off qmail bounced email testing\n\
-	--recursion-max <level> : Set the maximum recursion level to 'level'\n\
-	--timeout <seconds>	: Set the maximum number of CPU seconds ripMIME can run for\n\
-	\n\
-	--debug : Produces detailed information about the whole decoding process\n\
-	--extended-errors:	Produces more return codes, even for non-fatals\n\
-	-V --version : Give version information\n\
-	--buildcodes : Give the build information (tstamp, date and system information)\n\
-	-h : This message (help)\n\n\n";
+char version[] = "v1.4.0.6 - December 12, 2005 (C) PLDaniels http://www.pldaniels.com/ripmime";
+char help[] = "ripMIME -i <mime file> -d <directory>"
+	"[-p prefix] [-e [header file]] [-vVh] [--version]"
+	"[--no_nameless] [--unique_names [--prefix|--postfix|--infix]]"
+	"[--paranoid] [--mailbox] [--formdata] [--debug]"
+	"[--no-tnef] [--no-quotedprintable] [--no-uudecode]\n"
+	"Options available :\n"
+	"-i : Input MIME encoded file (use '-' to input from STDIN)\n"
+	"\tIf <mime file> is a directory, it will be recursed\n"
+	"-d : Output directory\n"
+	"-p : Specify prefix filename to be used on files without a filename (default 'text')\n"
+	"-e [headers file name] : Dump headers from mailpack (default '_headers_')\n"
+	"-v : Turn on verbosity\n"
+	"-q : Run quietly, do no report non-fatal errors\n"
+	"\n"
+	"--verbose-contenttype : Turn on verbosity of file content type\n"
+	"--verbose-oldstyle : Uses the v1.2.x style or filename reporting\n"
+	"--verbose-defects: Display a summary of defects in the email\n"
+	"--verbose-mime: Displays 'Email is MIME' if the email is MIME\n"
+	"--stdout : All reporting goes to stdout (Default)\n"
+	"--stderr : All reporting goes to stderr\n"
+	"--syslog : All reporting goes to syslog\n"
+	"\n"
+	"--no-paranoid : [ Deprecated ] Turns off strict ascii-alnum filenaming\n"
+	"--paranoid: Converts all filenames to strict 7-bit compliance\n"
+	"--name-by-type: Saves a given attachment by its content-type if it has no other name\n"
+	"--no-nameless : Do not save nameless attachments\n"
+	"--overwrite : Overwrite files if they have the same name on extraction\n"
+	"--unique-names : Dont overwrite existing files (This is the default behaviour)\n"
+	"--prefix : rename by putting unique code at the front of the filename\n"
+	"--postfix : rename by putting unique code at the end of the filename\n"
+	"--infix : rename by putting unique code in the middle of the filename\n"
+	"\n"
+	"--mailbox : Process mailbox file\n"
+	"--formdata : Process as form data (from HTML form etc)\n"
+	"\n"
+	"--no-tnef : Turn off TNEF/winmail.dat decoding\n"
+	"--no-ole : Turn off OLE decoding\n"
+	"--no-uudecode : Turns off the facility of detecting UUencoded attachments in emails\n"
+	"--no-quotedprintable : Turns off the facility of decoding QuotedPrintable data\n"
+	"--no-doublecr : Turns off saving of double-CR embedded data\n"
+	"--no-mht : Turns off MHT (a Microsoft mailpack attachment format ) decoding\n"
+	"--no-multiple-filenames : Turns off the multiple filename exploit handling\n"
+	"\n"
+	"--disable-qmail-bounce : Turns off qmail bounced email testing\n"
+	"--recursion-max <level> : Set the maximum recursion level to 'level'\n"
+	"--timeout <seconds>	: Set the maximum number of CPU seconds ripMIME can run for\n"
+	"\n"
+	"--debug : Produces detailed information about the whole decoding process\n"
+	"--extended-errors:	Produces more return codes, even for non-fatals\n"
+	"-V --version : Give version information\n"
+	"--buildcodes : Give the build information (tstamp, date and system information)\n"
+	"-h : This message (help)\n\n\n";
 
 /*-----------------------------------------------------------------\
  Function Name	: RIPMIME_report_filename_decoded
@@ -260,6 +264,7 @@ int RIPMIME_parse_parameters (struct RIPMIME_globals *glb, int argc, char **argv
 
 				case 'v':
 					MIME_set_verbosity (1);
+					glb->verbose = 1;
 					break;
 
 				case 'q':
@@ -279,17 +284,16 @@ int RIPMIME_parse_parameters (struct RIPMIME_globals *glb, int argc, char **argv
 					// if we get ANOTHER - symbol, then we have an extended flag
 
 				case '-':
-					if (strncmp
-							(&(argv[i][2]), "verbose-contenttype",
-							 strlen ("verbose-contenttype")) == 0)
-					{
+					if (strncmp (&(argv[i][2]), "verbose-contenttype", strlen ("verbose-contenttype")) == 0) {
 						MIME_set_verbosity_contenttype (1);
-					}
-					else
-						if (strncmp
-								(&(argv[i][2]), "verbose-oldstyle",
-								 strlen ("verbose-oldstyle")) == 0)
-						{
+
+					/** 20051117-0927:PLD: 
+					 ** If client uses --verbose-mime, then make ripMIME report when it
+					 ** detects a MIME encoded email **/
+					} else if (strncmp(&(argv[i][2]), "verbose-mime", strlen("verbose-mime"))==0) {
+						MIME_set_report_MIME(1);
+
+					} else if (strncmp (&(argv[i][2]), "verbose-oldstyle", strlen ("verbose-oldstyle")) == 0) {
 							MIME_set_verbosity_12x_style (1);
 							MIME_set_filename_report_fn (NULL);
 						}
@@ -513,6 +517,7 @@ int RIPMIME_init (struct RIPMIME_globals *glb)
 	glb->timeout = 0;
 	glb->quiet = 0;
 	glb->verbose_defects = 0;
+	glb->verbose = 0;
 
 	return 0;
 }
@@ -539,6 +544,113 @@ void RIPMIME_signal_alarm( int sig )
 }
 
 
+/*-----------------------------------------------------------------\
+ Function Name	: RIPMIME_unpack_single
+ Returns Type	: int
+ 	----Parameter List
+	1. struct RIPMIME_globals *glb, 
+	2.  char *fname , 
+ 	------------------
+ Exit Codes	: 
+ Side Effects	: 
+--------------------------------------------------------------------
+ Comments:
+ 
+--------------------------------------------------------------------
+ Changes:
+ 
+\------------------------------------------------------------------*/
+int RIPMIME_unpack_single( struct RIPMIME_globals *glb, char *fname )
+{
+	int result = 0;
+
+	/** Set the alarm timeout feature. **/
+	if (glb->timeout > 0) {
+		signal(SIGALRM, RIPMIME_signal_alarm);
+		alarm(glb->timeout);
+	}
+
+	MIMEH_set_outputdir (glb->dir);
+	result = MIME_unpack (glb->dir, fname, 0);
+
+	// do any last minute things
+
+	MIME_close ();
+
+	return result;
+}
+
+/*-----------------------------------------------------------------\
+ Function Name	: RIPMIME_unpack
+ Returns Type	: int
+ 	----Parameter List
+	1. struct RIPMIME_globals *glb , 
+ 	------------------
+ Exit Codes	: 
+ Side Effects	: 
+--------------------------------------------------------------------
+ Comments:
+ 
+--------------------------------------------------------------------
+ Changes:
+ 
+\------------------------------------------------------------------*/
+int RIPMIME_unpack( struct RIPMIME_globals *glb )
+{
+	struct stat st;
+	int stat_result = 0;
+	int result = 0;
+	int input_is_directory = 0;
+
+	/** If we're not inputting from STDIN, check to see if the
+	  ** input is a directory **/
+	if (strcmp(glb->inputfile,"-")!=0) {
+		stat_result = stat(glb->inputfile, &st);
+		if (stat_result != 0) return -1;
+
+		if (S_ISDIR(st.st_mode)) input_is_directory = 1;
+	}
+
+	if (input_is_directory == 1) {
+		/** Unpack all files in directory **/
+		DIR *dir;
+		struct dirent *dir_entry;
+
+		fprintf(stderr,"input file is a directory, recursing\n");
+		dir = opendir(glb->inputfile);
+		if (dir == NULL) return -1;
+		
+		do {
+			/** Check every entry in the directory provided and if it's a file
+			  ** try to unpack it **/
+			char fullfilename[1024];
+
+			dir_entry = readdir( dir );
+			if (dir_entry == NULL) break;
+
+			if (strcmp(dir_entry->d_name, ".")==0) continue;
+			if (strcmp(dir_entry->d_name, "..")==0) continue;
+
+			snprintf(fullfilename,sizeof(fullfilename),"%s/%s", glb->inputfile, dir_entry->d_name);
+			stat_result = stat( fullfilename, &st );
+			if (stat_result != 0) continue;
+			if (S_ISREG(st.st_mode)) {
+				/** If the directory entry is a file, then unpack it **/
+				fprintf(stdout,"Unpacking mailpack %s\n",fullfilename);
+				result = RIPMIME_unpack_single(glb, fullfilename);
+			}
+		} while (dir_entry != NULL); /** While more files in the directory **/
+
+		closedir( dir );
+
+	} else {
+		/** If the supplied file was actually a normal file, then decode normally **/
+		result = RIPMIME_unpack_single( glb, glb->inputfile );
+	}
+
+	return result;
+}
+
 /*-----------------------------------------------------------------\
  Function Name	: main
  Returns Type	: int
@@ -595,12 +707,6 @@ int main (int argc, char **argv)
 
 	RIPMIME_parse_parameters (&glb, argc, argv);
 
-	// Set the alarm timeout feature.
-	if (glb.timeout > 0)
-	{
-		signal(SIGALRM, RIPMIME_signal_alarm);
-		alarm(glb.timeout);
-	}
 
 	// if our input filename wasn't specified, then we better let the user know!
 	if (!glb.inputfile)
@@ -641,13 +747,8 @@ int main (int argc, char **argv)
 	}
 
 	// Unpack the contents
+	RIPMIME_unpack(&glb);
 
-	MIMEH_set_outputdir (glb.dir);
-	result = MIME_unpack (glb.dir, glb.inputfile, 0);
-
-	// do any last minute things
-
-	MIME_close ();
 
 
 	// Possible exit codes include;
