@@ -1,6 +1,5 @@
 /*
- *
- * P3Scan v2.1
+ * P3Scan v2.3.2
  *
  * (C) 2003-2005 by Jack S. Lai <laitcg@cox.net>
  *
@@ -12,7 +11,8 @@
  * email environment, and many other changes.
  *
  * The initial release of p3scan includes patches made and submitted to the
- * original project but were never incorporated.
+ * original project but were never incorporated. Please see the README for
+ * further information.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,6 +28,10 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
+ * This program is released under the GPL with the additional exemption that
+ * compiling, linking, and/or using OpenSSL is allowed."
+ * (http://www.openssl.org/support/faq.html#LEGAL2)
+ *
  */
 
 #ifndef _SCANNER_H
@@ -37,13 +41,19 @@
 
 extern scanner_t scanner_basic;
 extern scanner_t scanner_avpd;
+extern scanner_t scanner_avpd_new;
 extern scanner_t scanner_trophie;
+extern scanner_t scanner_clamd;
+extern scanner_t scanner_bash;
 
 scanner_t *scannerlist[] = {
-    &scanner_basic, /* first one is default */
-    &scanner_avpd,
-    &scanner_trophie,
-    NULL
+   &scanner_basic, /* first one is default */
+   &scanner_avpd,
+   &scanner_avpd_new,
+   &scanner_trophie,
+   &scanner_clamd,
+   &scanner_bash,
+   NULL
 };
 
 #endif
