@@ -1,8 +1,7 @@
 /*
- * 
- * P3Scan v1.0
+ * P3Scan v2.3.2
  *
- * (C) 2003 by Jack S. Lai <laitcg@cox.net>
+ * (C) 2003-2005 by Jack S. Lai <laitcg@cox.net>
  *
  * It's intent is to provide a follow on program to POP3-Virusscan-Proxy 0.4
  * by Folke Ashberg <folke@ashberg.de>.
@@ -12,7 +11,8 @@
  * email environment, and many other changes.
  *
  * The initial release of p3scan includes patches made and submitted to the
- * original project but were never incorporated.
+ * original project but were never incorporated. Please see the README for
+ * further information.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,6 +28,10 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
+ * This program is released under the GPL with the additional exemption that
+ * compiling, linking, and/or using OpenSSL is allowed."
+ * (http://www.openssl.org/support/faq.html#LEGAL2)
+ *
  */
 
 #include <stdio.h>
@@ -52,6 +56,9 @@
 
 struct configuration_t * config;
 
+extern void * w_malloc(size_t bytes);
+extern void w_free(void *f_address);
+
 typedef unsigned long ULONG;
 #define FL_GETVERSION  0x04
 
@@ -139,14 +146,16 @@ static int avp_sendcommand(struct proxycontext * p,
          if ((uintbuf&0x100)!=0){ /* we got result string to read */
             if ((Rez=read(avp_fd,(char*)&uintbuf,sizeof(ULONG)))==-1) return -2;
             do_log(LOG_DEBUG, "Result string lenght: %d", uintbuf);
-            ResultBuf=(char*)malloc(uintbuf+1);
+            //--ResultBuf=(char*)malloc(uintbuf+1);
+            ResultBuf=(char*)w_malloc(uintbuf+1);
             if(ResultBuf!=NULL){
                char *ResultStr=ResultBuf;
                ResultBuf[0]=0;
                //if((Rez=recv(avp_fd,ResultStr,uintbuf,0))==-1) return -2;
                while((uintbuf>0)&&((Rez=recv(avp_fd,ResultStr,uintbuf,0))!=0)){
                   if(Rez==-2){
-                     free(ResultBuf);
+                     //--free(ResultBuf);
+                     w_free(ResultBuf);
                      return -2;
                   } else {
                      uintbuf-=Rez;
@@ -188,7 +197,8 @@ static int avp_sendcommand(struct proxycontext * p,
       case 0xf:
          {
          do_log(LOG_CRIT, "AVPD want's to disinfect! Please tell him not to do.");
-         free(ResultBuf);
+         //--free(ResultBuf);
+         w_free(ResultBuf);
          return -3;
          }
       default:
@@ -225,7 +235,8 @@ static int avp_scanfile(struct proxycontext * p, int flags, char * filetoscan, c
 
    // build the message
    len=strlen(filetoscan)+30;
-   tbuf=malloc(len+1);
+   //--tbuf=malloc(len+1);
+   tbuf=w_malloc(len+1);
    (void)time(&now);
    (void)snprintf(tbuf, len, "<%d>%.15s:%s", flags, ctime(&now)+4, filetoscan);
 
@@ -242,7 +253,8 @@ static int avp_scanfile(struct proxycontext * p, int flags, char * filetoscan, c
          do_log(LOG_CRIT, "Error occured during avpd conversation");
          break;
    }
-   free(tbuf);
+   //--free(tbuf);
+   w_free(tbuf);
    if (virinfo){
       /* process virinfo */
       /* format is: <filename>    infected: EICAR-Test-File */
@@ -261,7 +273,8 @@ static int avp_scanfile(struct proxycontext * p, int flags, char * filetoscan, c
       while ((len=strlen(v))>0 && isspace(v[len-1])) v[len-1]='\0';
       do_log(LOG_DEBUG, "virinfo: '%s'", v);
       *virname=strdup(v);
-      free(virinfo);
+      //--free(virinfo);
+      w_free(virinfo);
    } else *virname=NULL;
 
    return rez;
@@ -276,8 +289,10 @@ static int init1(void){
    }
    len=strlen(config->virusscanner);
    /* Build the Nodes */
-   if ((NodeCtl=malloc(len + strlen(AVP_NODE_CTL) + 10))==NULL) return -1;
-   if ((NodePid=malloc(len + strlen(AVP_NODE_PID) + 10))==NULL) return -1;
+   //--if ((NodeCtl=malloc(len + strlen(AVP_NODE_CTL) + 10))==NULL) return -1;
+   //--if ((NodePid=malloc(len + strlen(AVP_NODE_PID) + 10))==NULL) return -1;
+   if ((NodeCtl=w_malloc(len + strlen(AVP_NODE_CTL) + 10))==NULL) return -1;
+   if ((NodePid=w_malloc(len + strlen(AVP_NODE_PID) + 10))==NULL) return -1;
    strncpy(NodeCtl, config->virusscanner, len + 1);
    if (config->virusscanner[len-1]!='/') strcat(NodeCtl, "/");
    strncpy(NodePid, NodeCtl, strlen(NodeCtl) + 1);
