@@ -1,6 +1,5 @@
 /*
- *
- * P3Scan v2.1
+ * P3Scan v2.3.2
  *
  * (C) 2003-2005 by Jack S. Lai <laitcg@cox.net>
  *
@@ -12,7 +11,8 @@
  * email environment, and many other changes.
  *
  * The initial release of p3scan includes patches made and submitted to the
- * original project but were never incorporated.
+ * original project but were never incorporated. Please see the README for
+ * further information.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,6 +28,10 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
+ * This program is released under the GPL with the additional exemption that
+ * compiling, linking, and/or using OpenSSL is allowed."
+ * (http://www.openssl.org/support/faq.html#LEGAL2)
+ *
  */
 
 #include <stdio.h>
@@ -43,6 +47,8 @@
 extern int checkbuff(int fdc);
 extern int checktimeout(struct proxycontext *p);
 struct configuration_t * config;
+extern void * w_malloc(size_t bytes);
+extern void w_free(void *f_address);
 
 static int scan(struct proxycontext *p, char ** virname){
    int ret,ret2,fdc;
@@ -55,7 +61,7 @@ static int scan(struct proxycontext *p, char ** virname){
    int pcre_erroffset;
    int offsets[50];
 #define VISIZE 1000
-   char *vi=malloc(VISIZE);
+   char *vi=w_malloc(VISIZE);
    int vipos = 0;
    int vcodndx;
 
@@ -76,19 +82,14 @@ static int scan(struct proxycontext *p, char ** virname){
    }
 
    len=strlen(config->virusscanner) + strlen(p->scanthis) + 1 + strlen(" '' 2>&1");
-   command=malloc(len+1);
-   if (command==NULL){
-      do_log(LOG_EMERG, "Cant allocate memory for command!");
-      free(vi);
-      return SCANNER_RET_ERR;
-   }
+   command=w_malloc(len+1);
    snprintf(command, len, "%s '%s' 2>&1", config->virusscanner, p->scanthis);
    do_log(LOG_DEBUG, "popen %s", command);
 
    if ((scanner=popen(command, "r"))==NULL){
       do_log(LOG_ALERT, "Can't start scanner '%s' !!!", command);
-      free(vi);
-      free(command);
+      w_free(vi);
+      w_free(command);
       return SCANNER_RET_ERR;
    }
    fdc=fileno(scanner);
@@ -104,7 +105,9 @@ static int scan(struct proxycontext *p, char ** virname){
    *virname=vi;
    while ((fgets(line, 4095, scanner))!=NULL){
       line[strlen(line)-1]='\0';
+#ifdef DEBUG_SCANNING
       do_log(LOG_DEBUG, "ScannerLine: '%s'", line);
+#endif
       if (rx){
          ret = pcre_exec(rx, NULL, line, strlen(line), 0, 0, offsets, 50);
          if (ret > config->virusregexpsub){
@@ -118,33 +121,34 @@ static int scan(struct proxycontext *p, char ** virname){
       }
    }
    ret=pclose(scanner);
-   free(command);
+   //--free(command);
+   w_free(command);
    if (vipos > 3) vi[vipos-3]='\0';
    do_log(LOG_DEBUG, "vi : '%s'", vi);
    if (rx) pcre_free(rx);
 
    if (!WIFEXITED(ret)){
-      do_log(LOG_ALERT, "Scanner returned abnormal signal (%i)", ret);
-      free(vi);
-      return SCANNER_RET_ERR;
+     do_log(LOG_ALERT, "Scanner returned abnormal signal (%i)", ret);
+     return SCANNER_RET_ERR;
    } else
-   do_log(LOG_DEBUG, "Scanner returned signal %i", WEXITSTATUS(ret));
-   ret=WEXITSTATUS(ret);
-   for (vcodndx = 0; vcodndx < MAX_VIRUS_CODES && config->viruscode[vcodndx] != -1; vcodndx++) {
-      if (ret == config->viruscode[vcodndx]){
-         do_log(LOG_DEBUG, "Message contains a virus!");
-         //free(vi);
-         return SCANNER_RET_VIRUS; /* contains a virus */
-      }
-   }
-   if (ret!=0){
-      do_log(LOG_ALERT,"WARNING: Your scanner returned neither 0 nor a viruscode, but %i",ret);
-      // free(vi);
-      return SCANNER_RET_ERR;
-   }
-   do_log(LOG_DEBUG, "Basic scanner says goodbye");
-   //free(vi);
-   return SCANNER_RET_OK; /* all ok, no virus */
+     do_log(LOG_DEBUG, "Scanner returned signal %i", WEXITSTATUS(ret));
+     ret=WEXITSTATUS(ret);
+     for (vcodndx = 0; vcodndx < MAX_VIRUS_CODES && config->viruscode[vcodndx] != -1; vcodndx++) {
+       if (ret == config->viruscode[vcodndx]) return SCANNER_RET_VIRUS; /* contains a virus */
+     }
+     for (vcodndx = 0; vcodndx < MAX_VIRUS_CODES && config->gvcode[vcodndx] != -1; vcodndx++) {
+       if (ret == config->gvcode[vcodndx]){
+         ret = 0;
+         do_log(LOG_DEBUG, "Basic scanner says goodbye (goodcode)");
+         return SCANNER_RET_OK; /* good return code */
+       }
+     }
+     if (ret!=0){
+       do_log(LOG_ALERT, "WARNING: Your scanner returned neither 0, a viruscode, nor a good viruscode, but %i", ret);
+       return SCANNER_RET_ERR;
+     }
+     do_log(LOG_DEBUG, "Basic scanner says goodbye");
+     return SCANNER_RET_OK; /* all ok, no virus */
 }
 
 static int init1(void){
